function Y = SRencode(x,FMT)

% SRencode SR (Stack-Run) run-length encoder, encode the length of zeros and value of
% non-zero elements. [Tsai'96]
% 
%
% 
% The SRencode takes input of an array contains zeros and non-zero values
% outputs a string array consists of four symbols: +,-,0,1
%
% all non-zero values are first incremented (or decremented) by 1.
% 0,1 are used to describe the LSB of the non-zero values in binary.
% +,- are used to describe the MSB and sign of non-zero values in binary
% 
%
% + is 1 and - is 0 in describing the run-length of zeros in binary.
% the MSB (since it's always 1) is omitted, except when the run length is
% 2^k-1, where k is an integer. 
%
%
% INPUT: 'x' e.g. quantization levels, array in double. 
%        FMT is a string identifies the output format, can be 'char',
%        or 'double'. default format is 'double'. 
% OUTPUT: Y is the output array. when FMT is NOT 'char', '+' becomes 3 and
% '-' becomes 2. 
%
% Example: x = [-11 0 0 0 34 0 0 0 0 2 1 0 0 0 5];
%          y = SRencode(x,'char')
%          y = -100+++00011--+1+0+++10
%
%

% %%%%%%%%%%%%%%%%%%%%%%%%
% by A. Kim
% date: 28.02.2011
% last change: 24.09.2011
% %%%%%%%%%%%%%%%%%%%%%%%%%%


%pre allocate codeword size. -1 is dummy bit. 
y = -ones(size(x));

sizey1 = 1; %starting position of the codeword
sizey2 = 0; %end position.

while ~isempty(x) % check if x has any value
    
    if isempty(find(x~=0, 1))% x has only zeros.
    
         RLbin_str = int2bit(length(x));
       
        % checking to see if run-length is 2^k-1 
        if round(log2(length(x)+1))-log2(length(x)+1)~=0
        
            RLbin_str = RLbin_str(2:length(RLbin_str));
        
        end
        
        % mapping to '+','-'. where '+' is 3, '-' is 2.
        RLbin_str = RLbin_str +2;
    
        
        %determine end position of codeword
        sizey2 = sizey2+length(RLbin_str);
        
        % remove the encoded zeros.
        x = [];
        
        % assign to output and update starting position of codeword
        y(sizey1:sizey2)=RLbin_str;
        sizey1 = sizey2+1;
        
        
    else % x has both zero and nonzero elements
            
           
            if x(1)==0  % when first element is zero, code run length:
                
                % determine length of zeros, convert to binary.
                marker = length(x(1:find(x~=0, 1 )-1));
                RLbin_str = int2bit(marker);
                
                % checking to see if run-length is 2^k-1,if not remove MSB 
                if round(log2(marker+1))-log2(marker+1)~=0
        
                    RLbin_str = RLbin_str(2:length(RLbin_str));
                    
                end
                
                %change 0, to -, 1 to +
                
                RLbin_str = RLbin_str +2;
    
                
                %update codeword start and end positions and output array
                sizey2 = sizey2+length(RLbin_str);
                
                y(sizey1:sizey2)=RLbin_str;
                sizey1 = sizey2+1;
                
                
                
                % remove the encoded zeros from input array
                x = x(find(x~=0,1):length(x));
                
                
                
            else % encode the first nonzero value
    
                
                % first increment the absolute value by 1. retain sign.
                nz_val = abs(x(1))+1;
                
                %NZbin_str = dec2bin(abs(nz_val));
                NZbin_str = int2bit(nz_val);
                
                % change the MSB into '+' or '-'
                if sign(x(1))>0
                    NZbin_str(1)=3; % '+' is 3
                else
                    
                    NZbin_str(1)=2;% '-' is 2;
                end
                
                % update codeword positions and output array
                sizey2 = sizey2+length(NZbin_str);
                y(sizey1:sizey2)=NZbin_str;
                sizey1 = sizey2+1;
                
                
                
                % remove the nonzero value from input array
                if length(x)==1  % come to the last element
                    x = [];
                else 
                    x = x(2:length(x));
                    
                end
                
                
            end
            
    end
   
     
    
    
    
end

% remove the dummy bits -1;

y = y(y>=0);

if nargin==1 || strcmp(FMT,'double')==1
    Y = y;
    
elseif  strcmp(FMT, 'char')==1
                
    Y(y==3) = '+';
    Y(y==2) = '-';
    Y(y==0) = '0';
    Y(y==1) = '1';

else
    fprintf('ERROR: unknown output format.');
    Y = [];
end
    